<?php

class DocsBackendFestiPlugin extends DocsFestiPlugin
{
    protected $_menuOptions = array(
        'settings' => "Settings",
        'help'     => "Shortcodes",
    );
    
    protected $_defaultMenuOption = 'settings';
    
    protected function onInit()
    {       
        $this->addActionListener('admin_menu', 'onAdminMenuAction');
        
        $this->addFilterListener('post_type_link', 'documentsLinkFilter');   
             
        $this->addFilterListener('term_link', 'versionsLinkFilter');
       
        $this->addActionListener(
            'add_meta_boxes',
            'onInitAccessParametersAction'
        );
        
        $this->addActionListener('save_post', 'onSaveAccessParametersAction');
        
        $this->addActionListener(
            'admin_enqueue_scripts',
            'checkShortcodesAction'
        );
        
        $this->addActionListener(
            'admin_print_styles',
            'onInitCssAction'
        );
        
        $this->addActionListener(
            'admin_print_styles',
            'onInitJsAction'
        );

        $this->addActionListener(
            'admin_init',
            'appendButtonToWordpressVisualEditorAction'
        );

        $this->addActionListener(
            'admin_enqueue_scripts',
            'doCreateObjectOfVersionsForShortcodeButtonAction'
        );
    } // end onInit
    
    public function doCreateObjectOfVersionsForShortcodeButtonAction()
    {
        if (!$this->_isPluginPostType()) {
            return false;
        }
        
        $this->addActionListener(
            'admin_head',
            'appendScriptWithObjectForShortcodeButton'
        );
    } // end appendScriptWithObjectForShortcodeButton
    
    public function appendScriptWithObjectForShortcodeButton()
    {
        $versionsList = $this->getAllVersions();
        
        $versionsData = array();
        
        foreach ($versionsList as $value) {
            $key = $value->term_id;
            $versionsData[$key]['id'] = $key;
            $versionsData[$key]['name'] =  $value->name;
        }
        
        $vars = array(
             'versions' => json_encode($versionsData)
        );
        
        echo $this->fetch('versions_array.phtml', $vars);
    } // end appendScriptWithObjectForShortcodeButton
    
    public function appendButtonToWordpressVisualEditorAction()
    {
        $this->addFilterListener(
            'mce_external_plugins',
            'addTinymcePluginFilter'
        );
        
        $this->addFilterListener(
            'mce_buttons',
            'onRegisterButtonFilter'
        );       
    } // end appendButtonToWordpressVisualEditorAction
    
    public function addTinymcePluginFilter($plugin)
    {
        if (!$this->_isPluginPostType()) {
            return $plugin;
        }

        $plugin['festiDocs'] = $this->getPluginJsUrl('versions_button.js');
        $plugin['festiDocsNavigation'] = $this->getPluginJsUrl(
            'custom_button.js'
        );
        return $plugin;
    } //end addTinymcePluginFilter
    
    public function onRegisterButtonFilter($buttons)
    {
        if (!$this->_isPluginPostType()) {
            return $buttons;
        }
        
        array_push($buttons, "festiDocsButton");
        array_push($buttons, "festiDocsNavigationButton");
        return $buttons;
    } // end onRegisterButtonFilter
    
    public function getPluginCssUrl($fileName) 
    {
        return $this->_pluginCssUrl.'backend/'.$fileName;
    } // end getPluginCssUrl
    
    public function getPluginJsUrl($fileName)
    {
        return $this->_pluginJsUrl.'backend/'.$fileName;
    } // end getPluginJsUrl
    
    public function onInitCssAction()
    {
        if (!$this->_isPluginPostType() && !$this->_hasSettingsInRequest()) {
            return false;
        }

        $this->onEnqueueCssFileAction(
            'festi-docs-styles',
            'style.css'
        );
        
        $this->onEnqueueCssFileAction(
            'festi-docs-menu',
            'menu.css'
        );
    } // end onInitCssAction
    
    public function onInitJsAction()
    {
        if (!$this->_isPluginPostType() && !$this->_hasSettingsInRequest()) {
            return false;
        }
        
        $this->onEnqueueJsFileAction('jquery');
        $this->onEnqueueJsFileAction(
            'festi-docs-general',
            'general.js',
            'jquery'
        );        
    } // end onInitJsAction
    
    public function onInstall($refresh = false, $settings = false)
    {        
        if (!$this->_fileSystem) {
            $this->_fileSystem = $this->getFileSystemInstance();
        }
        
        if ($this->_hasPermissionToCreateCacheFolder()) {
            $this->_fileSystem->mkdir($this->_pluginCachePath, 0777);
        }
        
        if (!$refresh) {
            $settings = $this->getOptions('settings');    
        }
              
        if (!$refresh && !$settings) {
            $this->_doInitDefaultOptions('settings');
        }

        if (!$this->hasFrontendPageInWordpress()) {
            $result = $this->_doCreateFrontendPageForDisplayContent();
                    
            if ($refresh && $result) {
                echo $this->fetch('redirect_to_settings.phtml');
            }  
        }       
    } // end onInstal
    
    private function _doInitDefaultOptions($option, $instance = NULL)
    {
        $methodName = $this->getMethodName('load', $option);
        
        if (is_null($instance)) {
            $instance = $this;
        }

        $method = array($instance, $methodName);
        
        if (!is_callable($method)) {
            throw new Exception("Undefined method name: ".$methodName);
        }

        $options = call_user_func_array($method, array());
        foreach ($options as $ident => &$item) {
            if ($this->_hasDefaultValueInItem($item)) {
                $values[$ident] = $item['default'];
            }
        }
        unset($item);
        
        $this->updateOptions($option, $values);
    } // end _doInitDefaultOptions
    
    private function _hasDefaultValueInItem($item)
    {
        return isset($item['default']);
    } //end _hasDefaultValueInItem
    
    public function getMethodName($prefix, $option)
    {
        $option = explode('_', $option);
        
        $option = array_map('ucfirst', $option);
        
        $option = implode('', $option);
        
        $methodName = $prefix.$option;
        
        return $methodName;
    } // end getMethodName
    
    private function _hasSettingsInRequest()
    {
        return array_key_exists('page', $_GET)
               && $_GET['page'] == 'festi-docs';
    } // end _hasSettingsInRequest
    
    public function checkShortcodesAction() 
    {
        if (!$this->_isDocumentEditPage()) {
           return false;
        }
        
        $content = $this->getDocumentContent();
        $versionsNames = $this->getAllVersionsNamesFromShortcode();
        
        if(!$versionsNames) {
            return false;
        }
        
        $activeVersions = $this->getActiveVersions($_GET['post']);
        
        $errors = array();
        foreach ($versionsNames as $name) {
             $version = $this->getVersionOptionByField(
                    'name',
                    $name,
                    'term_id'
            );

            if (!$version) {
               $errors[] = 'AllVersions';
            } 
            
            if (!in_array($version, $activeVersions) && $activeVersions) {
                 $errors[] = 'ActiveVersions';
            }
        }
                    
        if (!$errors) {
            return false;
        }
        
        if (in_array('AllVersions', $errors)) {
            $this->displayContentEroror(
                'onDisplayAllVersionsErorrAction'
            );
        }
        
        if (in_array('ActiveVersions', $errors)) {
            $this->displayContentEroror(
                'onDisplayActiveVersionsErorrAction'
            );
        }
    } // end checkShortcodesAction
    
    public function displayContentEroror($method)
    {
        $this->addActionListener(
            'admin_notices',
            $method 
        );
    } // end displayContentEroror
    
    public function onDisplayActiveVersionsErorrAction()
    {
        $message = __(
            'In shortcode used version, which not support in the document',
            $this->_languageDomain
        );        

        $this->displayError($message);
    } //end onDisplayActiveVersionsErorrAction

    public function onDisplayAllVersionsErorrAction()
    {        
        $message = __(
            'In shortcode used nonexistent version. '.
            'Please create this version as new or change shortcode',
            $this->_languageDomain
        );
        
        $this->displayError($message);
    } //end onDisplayAllVersionsErorrAction
    
    public function getDocumentContent()
    {
        $result = get_post($_GET['post'], ARRAY_A);
        return $result['post_content'];
    } // end getDocumentContent
    
    public function getAllVersionsNamesFromShortcode()
    {             
        $content = $this->getDocumentContent();
        $pattern = $this->getShortcodePattern();
        
        preg_match_all('/'. $pattern .'/uis', $content, $matches);
        
        if (!$matches) {
            return false;
        }
                
        $attributes = $matches[3];
        
        $versions = $this->getVersionsNamesFromAttributes($attributes);
       
        return $versions;
    } // end getAllVersionsNamesFromShortcode
    
    public function getShortcodePattern()
    {
        $pattern = '\[(\[?)';
        $pattern .= '(DocumentorSnippet)';
        $pattern .= '(?![\w-])([^\]\/]*(?:\/(?!\])[^\]\/]*)*?)(?:(\/)\]|\](?:';
        $pattern .='([^\[]*+(?:\[(?!\/\2\])[^\[]*+)*+)\[\/\2\])?)(\]?)';
        
        return $pattern;
    } // end getShortcodePattern
    
    public function getVersionsNamesFromAttributes($attributes)
    {
        $versions = array();
        
        foreach ($attributes as $value) {
            $result = shortcode_parse_atts($value);

            if (!$result || !$this->_hasVersionsInAttributes($result)) {
                continue;
            }
            $content = $result['versions'];
            $list = $this->getVersionsListFromShortcodeVariable($content);
            $versions = array_merge($versions, $list);
        }
        
        return $versions;
    } // end getVersionsNamesFromAttributes
    
    private function _hasVersionsInAttributes($attributes = array())
    {
        return array_key_exists('versions', $attributes);
    } // end _hasVersionsInAttributes
    
    public function getActiveVersions($documentId)
    {
        $result = wp_get_object_terms(
            $documentId,
            $this->_versions, 
            array('fields' => 'ids')
        );
        
        return $result;
    } // end getActiveVersions
    
    public function onInitAccessParametersAction() { 
        add_meta_box(
            'festi_docs_access_parameters',
            __('Access Parameters', $this->_languageDomain), 
            array($this, 'onDisplayAccessParametersBox'),
            $this->_documents,
            'side',
            'core'
        ); 
    } // end onInitAccessParametersAction
    
    public function onDisplayAccessParametersBox($post, $box)
    {
        $value = $this->getMetaOptions(
            $post->ID,
            'festi_docs_access_parameters',
            true
        );

        $item = array(
            'lable' => __('only registered', $this->_languageDomain)
        );
        
        $vars = array(
            'ident'=> 'festiUserAvaliable',
            'item' => $item
        );
        
        if ($value) {
            $vars['currentValue'] = 1;
        }
        
        echo $this->fetch('form/input_checkbox.phtml', $vars); 
    } // end onDisplayAccessParametersBox
    
    public function onSaveAccessParametersAction($postId) {

        if ($this->_hasUserAvaliableInRequest())
        {
            $data = $_POST['festiUserAvaliable'];    
        } else {
            $data = false;
        }

        $this->updateMetaOptions(
            $postId,
            'festi_docs_access_parameters',
            $data
        ); 
    } // end onSaveAccessParametersAction
    
    private function _hasUserAvaliableInRequest()
    {
        return array_key_exists('festiUserAvaliable', $_POST)
               && !empty($_POST['festiUserAvaliable']);
    } // end _hasUserAvaliableInRequest
    
    public function updateMetaOptions($postId, $metaId, $data)
    {
        update_post_meta($postId, '_'.$metaId.'_data', $data);
    } // end updateMetaOptions    
    
    public function documentsLinkFilter($link, $document, $leavename = false)
    {
        $postType = $document->post_type;
        
        if (!$this->_hasPluginPostTypeInRequest($postType)) {
            return $link;
        }
        
        $settings = $this->getOptions('settings');

        $page = $settings['page'];
        
        $slug = $document->post_name;
        
        $url = $this->getLink($page, false, $slug, true);
        return $url;
    } // end documentLinkFilter
    
    public function versionsLinkFilter($link, $version, $taxonomy = false)
    {
        $taxonomy = $version->taxonomy;
        
        if (!$this->_hasPluginTaxonomyInRequest($taxonomy)) {
            return $link;
        }
        
        $settings = $this->getOptions('settings');
        
        $page = $settings['page'];
        
        $slug = $version->slug;
        
        $url = $this->getLink($page, $slug, false);
        return $url;
    } // end categoryLinkFilter
    
    public function getPluginTemplatePath($fileName)
    {
        return $this->_pluginTemplatePath.'backend/'.$fileName;
    } // end getPluginTemplatePath
    
    public function _onFileSystemInstanceAction()
    {
        $this->_fileSystem = $this->getFileSystemInstance();
    } // end _onFileSystemInstanceAction
    
    private function _hasPermissionToCreateCacheFolder()
    {
        return ($this->_fileSystem->is_writable($this->_pluginPath)
               && !file_exists($this->_pluginCachePath));
    } // end _hasPermissionToCreateFolder
    
    private function _doCreateFrontendPageForDisplayContent($settings = array())
    {    
        $settings = $this->getOptions('settings');

        $args = array(
            'post_title'     => 'Documentation',
            'post_content'   => __(
                'Do not delete this page unless you are using '.
                '"WordPress Documentation Builder"',
                $this->_languageDomain
            ),
            'post_status'    => 'publish',
            'post_type'      => 'page',
            'post_name'      => 'documentor',
            'comment_status' => 'closed'
        );

        $pageId =  wp_insert_post($args);
        
        $settings['page'] = $pageId;
        
        $this->updateOptions('settings', $settings);
        
        return  $pageId;
    } // _doCreateFrontendPageForDisplayContent
    
            
    public function onAdminMenuAction() 
    {
        if (!$this->hasFrontendPageInWordpress()) {
            $this->doAppendItemToAdminMenu('menu');
            return false;
        } else {
            $this->doAppendItemToAdminMenu('submenu', 'Settings');
        }
    } // end onAdminMenuAction
    
    public function doAppendItemToAdminMenu($type, $caption = 'Documentation')
    {
        $args = array(
             'parent'     => 'edit.php?post_type=festi_docs_documents',
             'title'      => __('Documentation', $this->_languageDomain),
             'caption'    => __($caption, $this->_languageDomain),
             'capability' => 'manage_options',
             'slug'       => 'festi-docs',
             'method'     => array(&$this, 'onDisplayOptionPage'),
             'iconUrl'    => $this->getPluginImagesUrl('icon_16x16.png')    
        );
        
        if ($type == 'menu') {
            $page = $this->doAppendMenu($args);
        } else {
            $page = $this->doAppendSubMenu($args);
        }
        
        $this->addActionListener(
            'admin_print_styles-'.$page, 
            'onInitCssAction'
        );
        
        $this->addActionListener(
            'admin_print_scripts-'.$page, 
            'onInitJsAction'
        );
        
        $this->addActionListener(
            'admin_head-'.$page,
            '_onFileSystemInstanceAction'
        );
        
        return $page;
    } // end doAppendItemToAdminMenu
    
    public function doAppendMenu($args = array())
    {
        $page = add_menu_page(
            $args['title'], 
            $args['caption'], 
            $args['capability'], 
            $args['slug'], 
            $args['method'], 
            $args['iconUrl']
        );
        return $page;  
    } //doAppendMenu
    
    public function doAppendSubMenu($args = array())
    {
        $page = add_submenu_page(
            $args['parent'],
            $args['title'], 
            $args['caption'], 
            $args['capability'], 
            $args['slug'], 
            $args['method']
        );
        return $page;  
    } //doAppendMenu
    
    public function onDisplayOptionPage()
    {
        $menu = $this->fetch('menu.phtml');
        echo $menu;

        $methodName = 'fetchOptionPage';
        
        if ($this->hasOptionPageInRequest()) {
            $postfix = $_GET['tab'];
        } else {
            $postfix = $this->_defaultMenuOption;
        }
        $methodName.= ucfirst($postfix);
        
        $method = array(&$this, $methodName);
        
        if (!is_callable($method)) {
            throw new Exception("Undefined method name: ".$methodName);
        }
        
        call_user_func_array($method, array());
    } // end onDisplayOptionPage
    
    public function fetchOptionPageSettings()
    {
        $vars = array();

        if ($this->_isRefreshPlugin()) {
            $this->onRefreshPlugin();
        }
        
        if ($this->_isRefreshCompleted()) {
            $message = __(
                'Success refresh plugin',
                $this->_languageDomain
            );
            
            $this->displayUpdate($message);   
        }
        
        $break = $this->_displayPluginErrors();
        
        if ($break) {
            return false;
        }

        if ($this->isUpdateOptions('save')) {
            try {
                $this->_doUpdateOptions($_POST);
                           
                $message = __(
                    'Success update settings',
                    $this->_languageDomain
                );
                
                $this->displayUpdate($message);               
            } catch (Exception $e) {
                $message = $e->getMessage();
                $this->displayError($message);
            }
        }
        
        
        $options = $this->getOptions('settings');
        
        $vars['fieldset'] = $this->getOptionsFieldSet();        
        $vars['currentValues'] = $options;
        
        echo $this->fetch('settings_page.phtml', $vars);  
    } // end fetchOptionPageSettings
    
    private function _doUpdateOptions($newSettings = array())
    {
        $this->updateOptions('settings', $newSettings);
    } // end _doUpdateOptions
    
    public function getOptionsFieldSet()
    {
        $fildset = array(
            'general' => array(
                'legend' => __('General', $this->_languageDomain)
            )
        );
        
        $settings = $this->loadSettings();
        
        if ($settings) {
            foreach ($settings as $ident => &$item) {
                if (array_key_exists('fieldsetKey', $item)) {
                   $key = $item['fieldsetKey'];
                   $fildset[$key]['filds'][$ident] = $settings[$ident];
                }
            }
            unset($item);
        }
        
        return $fildset;
    } // end getOptionsFieldSet
    
    public function loadSettings()
    {
        $settings = array(
            'page' => array(
                'caption' => __('Frontend Page', $this->_languageDomain),
                'type' => 'text',
                'fieldsetKey' => 'general',
                'getPageUrl'  => true,
                'text' => ''
            ),
            'generalPageTitle' => array(
                'caption' => __(
                    'Project Name for General Page',
                    $this->_languageDomain
                ),
                'type' => 'input_text',
                'default' => 'Project',
                'fieldsetKey' => 'general',
            ),
            'docsDivider' => array(
                'caption' => __('Documents', $this->_languageDomain),
                'type'    => 'divider',
                'fieldsetKey' => 'general'
            ),
            'docsLocation' => array(
                'caption' => __('Location', $this->_languageDomain),
                'type' => 'select',
                'values' => array(
                    'top' => __('top', $this->_languageDomain),
                    'bottom' => __('bottom', $this->_languageDomain),
                ),
                'default' => 'top',
                'fieldsetKey' => 'general'
            ),
            'docsOrderBy' => array(
                'caption' => __('Sort by', $this->_languageDomain),
                'type' => 'select',
                'values' => array(
                    'ID' => __('id', $this->_languageDomain),
                    'title' => __('title', $this->_languageDomain),
                    'date' => __('date', $this->_languageDomain),
                    'modified' => __('modified', $this->_languageDomain),
                    'menu_order' => __('manual', $this->_languageDomain)
                ),
                'default' => 'ID',
                'fieldsetKey' => 'general'
            ),
            'docsOrder' => array(
                'caption' => __('Order', $this->_languageDomain),
                'type' => 'select',
                'values' => array(
                    'ASC' => __('ascending', $this->_languageDomain),
                    'DESC' => __('descending', $this->_languageDomain),
                ),
                'default' => 'ASC',
                'fieldsetKey' => 'general'
            ),
            'versionsDivider' => array(
                'caption' => __('Versions', $this->_languageDomain),
                'type'    => 'divider',
                'fieldsetKey' => 'general'
            ),
            'versionsOrderBy' => array(
                'caption' => __('Sort by', $this->_languageDomain),
                'type' => 'select',
                'values' => array(
                    'id' => __('id', $this->_languageDomain),
                    'name' => __('title', $this->_languageDomain)
                ),
                'default' => 'name',
                'fieldsetKey' => 'general'
            ),
            'versionsOrder' => array(
                'caption' => __('Order', $this->_languageDomain),
                'type' => 'select',
                'values' => array(
                    'ASC' => __('ascending', $this->_languageDomain),
                    'DESC' => __('descending', $this->_languageDomain),
                ),
                'default' => 'DESC',
                'fieldsetKey' => 'general'
            )
        );
        
        $values = $this->getOptions('settings');
        if ($values) {
            foreach ($settings as $ident => &$item) {
                if (array_key_exists($ident, $values)) {
                    $item['value'] = $values[$ident];
                }
            }
            unset($item);
        }
        
        return $settings;
    } // end loadSettings
    
    private function _isRefreshCompleted()
    {
        return array_key_exists('refresh_completed', $_GET);
    } // end _isRefreshCompleted
    
    public function isUpdateOptions($action)
    {
        return array_key_exists('__action', $_POST)
               && $_POST['__action'] == $action;
    } // end isUpdateOptions
    
    private function _displayPluginErrors($brake = false)
    {        
        $caheFolderErorr = $this->_detectTheCacheFolderAccessErrors();
        
        $frontendPageErorr = $this->_detectTheFrontendPageAccessErrors();
        
        if ($caheFolderErorr || $frontendPageErorr) {
            echo $this->fetch('refresh.phtml');
        }
        
        if ($frontendPageErorr) {
            $brake = true;
        }
        
        return $brake;
    } // end _displayPluginErrors
    
    private function _detectTheCacheFolderAccessErrors()
    {
        if (!$this->_fileSystem->is_writable($this->_pluginCachePath)) {

            $message = __(
                "Caching does not work! ",
                $this->_languageDomain
            );
            
            $message .= __(
                "You don't have permission to access: ",
                $this->_languageDomain
            );
            
            $path = $this->_pluginCachePath;
            
            if (!$this->_fileSystem->exists($path)) {
                $path = $this->_pluginPath;
            }
            
            $message .= $path;
            $message .= $this->fetch('manual_url.phtml');
            
            $this->displayError($message);
            
            return true;
        }
        
        return false;
    } // end _detectTheCacheFolderAccessErrors
    
    private function _detectTheFrontendPageAccessErrors()
    {
        if (!$this->hasFrontendPageInWordpress()) {

            $message = __(
                "No frontend page! ",
                $this->_languageDomain
            );
            
            $message .= __(
                "Click refresh to create a frontend page.",
                $this->_languageDomain
            );

            $message .= $this->fetch('manual_url.phtml');
            
            $this->displayError($message);
            
            return true;
        }
        
        return false;
    } // end _detectTheFrontendPageAccessErrors
    
    private function _isRefreshPlugin()
    {
        return array_key_exists('refresh_plugin', $_GET);
    } // end _isRefreshPlugin
    
    public function onRefreshPlugin()
    {
        $this->onInstall(true);
    } // end onRefreshPlugin
    
    protected function hasOptionPageInRequest()
    {
        return array_key_exists('tab', $_GET)
               && array_key_exists($_GET['tab'], $this->_menuOptions);
    } // end hasOptionPageInRequest
    
    public function redirectToFrontendPage()
    {
        $settings = $this->getOptions('settings');
        
        $vars = array(
            'url' => get_page_link($settings['page'])
        );
        
        echo $this->fetch('redirect_to_fronted_page.phtml', $vars);
    } // end redirectToFrontendPage
    
    public function addFilterListener($hook, $method)
    {
        add_filter($hook, array(&$this, $method), 10, 2);
    } // end addFilterListener

    private function _hasPluginPostTypeInRequest($postType)
    {
        return $postType == $this->_documents;
    } // end _hasPluginPostTypeInRequest
    
    private function _hasPluginTaxonomyInRequest($taxonomy)
    {
        return $taxonomy == $this->_versions;
    } // end _hasPluginPostTypeInRequest
    
    private function _isDocumentEditPage()
    {
        return $this->_isPluginPostType() 
               && array_key_exists('post', $_GET)
               && array_key_exists('action', $_GET)
               && $_GET['action'] == 'edit';
    } // end _isDocumentEditPage
    
    private function _isPluginPostType()
    {
        if (!$this->_hasWordpessPostTypeInGlobals()) {
            return false;
        }

        return $GLOBALS['post_type'] == 'festi_docs_documents';
    } // end _isPluginPostType
    
    private function _hasWordpessPostTypeInGlobals()
    {
        return array_key_exists('post_type', $GLOBALS);   
    } // end _hasWordpessPostTypeInGlobals
    
    public function fetchOptionPageHelp()
    {
        echo $this->fetch('help_page.phtml');
    } // end fetchOptionPageManual
}