<?php
class DocsFrontendFestiPlugin extends DocsFestiPlugin
{
    protected $_currentVersion = '';
    
    protected $_currentDocument = '';
    
    protected $_currentPage = '';
    
    protected $_allVersions = array();
    
    protected $_generalPageTitle = '';
    
    protected $_options = array();
    
    protected function onInit()
    {
        $this->addFilterListener(
            'the_posts',
            'onInitFrontendPageFilter',
            10,
            1
        );
    } // end onInit
    
    public function onInitFrontendPageFilter($posts)
    {
        $frontendPage = $this->getCurrentPage();
        
        foreach ($posts as $post) {
            if (!$this->_isFrontendPage($post->ID, $frontendPage)) {             
                continue;
            }
            
            $this->onInitCompatibleWithCrayonSyntaxHighlighter($frontendPage);
                    
            $this->onInitPage();
                
            $content = $this->getDocumentContent();

            if ($content) {
                $post->post_content = $content;   
            }
        }
        
        return $posts;
    } // end onInitFrontendPageFilter
    
    public function onInitCompatibleWithCrayonSyntaxHighlighter($page)
    {
        $result = $this->isPluginActive(
            'crayon-syntax-highlighter/crayon_wp.class.php'
        );
        
        if (!$result) {
            return false;
        }
        
        $options = $this->getAllPostsFromCrayonSyntaxHighlighterOption();

        if ($this->_hasPageInCrayonSyntaxHighlighterOption($options, $page)) {
            return true;
        }
        
        $this->appendPageToCrayonSyntaxHighlighterOption($options, $page);
    } // end onInitCompatibleWithCrayonSyntaxHighlighter
    
    public function appendPageToCrayonSyntaxHighlighterOption($options, $page)
    {
        if ($options) {
            array_push($options, $page);
        } else {
            $options = array($page);
        }

        update_option('crayon_posts', $options);   
    } // end appendPageToCrayonSyntaxHighlighterOption
    
    public function getAllPostsFromCrayonSyntaxHighlighterOption()
    {
       return get_option('crayon_posts');
    } // end getAllPostsFromCrayonSyntaxHighlighterOption
    
    private function _hasPageInCrayonSyntaxHighlighterOption($options, $page)
    {
        if (!$options) {
            return false;
        }
        
        return array_key_exists($page, $options);
    } // end _hasFrontendPageInCrayonSyntaxHighlighterOption
    
    private function _isFrontendPage($postId, $frontendPage)
    {
        return $frontendPage == $postId;
    } // end _isFrontendPage
    
    public function getPluginTemplatePath($fileName)
    {
        return $this->_pluginTemplatePath.'frontend/'.$fileName;
    } // end getPluginTemplatePath
    
    public function getPluginJsUrl($fileName)
    {
        return $this->_pluginJsUrl.'frontend/'.$fileName;
    } // end getPluginJsUrl
    
    public function onInitPage()
    {
        $this->onInitVariables();
        
        $this->addFilterListener(
            'the_content',
            'onDisplayContentAction',
            101,
            1
        );
        
        $this->addActionListener('wp_enqueue_scripts', 'onInitJsAction');
        
        $this->addActionListener('wp_print_styles', 'onInitCssAction');
        
        $this->addShortcodeListener(
            'DocumentorSnippet', 
            'onDisplayDocumentShortCode'
        );
        
        $this->addShortcodeListener(
            'DocumentorCustomButton',
            'onDisplayCustomButtonShortCode'
        );
    } // edn onInitPage
    
    public function getPluginCssUrl($fileName) 
    {
        return $this->_pluginCssUrl.'frontend/'.$fileName;
    } // end getPluginCssUrl
    
    public function onInitCssAction()
    {
        $this->onEnqueueCssFileAction(
            'festi-docs-custom-button',
            'custom_button.css'
        );
    } // end onInitCssAction
    
    public function onDisplayCustomButtonShortCode($variables = array())
    {
        $title = __("Button", $this->_languageDomain);
        
        $url = site_url();
        
        $target = '_self';
        
        if ($variables) {
            extract($variables);
        }

        $vars = array(
            'title'  => $title,
            'url'    => $url,
            'target' => $target
        );
        
        return $this->fetch("custom_button.phtml", $vars);   
    } // end onDisplayCustomButtonShortCode
    
    public function onDisplayDocumentShortCode($variables = array(), $content)
    {
        if (!$variables || !$this->_allVersions) {
            return $content;
        }

        extract($variables);
        
        if (!isset($versions) || $this->isContentOfCurrentVersion($versions)) {
            return $content;         
        }

        return false;
    } //end onDisplayDocumentShortCode
    
    public function isContentOfCurrentVersion($versions = array())
    {
        $versionsList = $this->getVersionsFromShortcodeVariable($versions);
        
        $result = in_array($this->_currentVersion, $versionsList);
        
        return $result;
    } // end isContentOfSelectedVersion
    
    public function getVersionsFromShortcodeVariable($versions)
    {
        $versions = $this->getVersionsListFromShortcodeVariable($versions);

        if (!$versions) {
            return false;
        }
        
        $versionsIdList = array();
        
        foreach ($versions as $value) {
            $versionId = $this->getVersionOptionByField(
                'name',
                $value,
                'term_id'
            );
            
            if ($versionId) {
                $versionsIdList[] = $versionId;
            }
        }
        
        $diff = array_diff($versionsIdList, $this->_allVersions);
        
        $versionsIdList = array_diff($versionsIdList, $diff);

        return $versionsIdList;
    } // end getVersionsFromShortcodeVariable
    
    public function onInitJsAction()
    {
        $this->onEnqueueJsFileAction('jquery');
        $this->onEnqueueJsFileAction(
            'festi-docs-general',
            'general.js',
            'jquery'
        );
    } // end onInitJsAction
    
    public function onInitVariables()
    {
        $this->_options = $this->getOptions('settings');
        
        $this->_currentPage = $this->getCurrentPage();
        
        $this->_allVersions = $this->getAllVersions(
            $this->_options['versionsOrderBy'],
            $this->_options['versionsOrder'],
            'ids'
        );
        
        $this->_currentVersion = $this->getCurrentVersion();
        
        if (!$this->_currentVersion && $this->hasVersionInRequest()) {
            $this->_doRedirect(false);
        }
        
        $this->_currentDocument = $this->getCurrentDocument();
        
        if (!$this->_isDocumentAvailableForTheCurrentVersion()) {
            $this->_doRedirect();
        }
        
        if ($this->_currentDocument) {
            $this->addFilterListener(
                'get_edit_post_link',
                'onDocumentEditLinkFilter',
                10,
                1
            );
        } else {
            $this->addFilterListener(
                'get_edit_post_link',
                'onRemoveDocumentEditLinkFilter',
                10,
                1
            );
        }
        
        $this->_generalPageTitle = $this->_options['generalPageTitle'];
    } // onInitVariables
    
    private function _isDocumentAvailableForTheCurrentVersion()
    {
        if ($this->hasPreviewInRequest()) {
            return true;
        }
        
        if (!$this->_currentDocument || !$this->_currentVersion) {
            return true;
        }
        
        $result = $this->_isAvailableForDisplay($this->_currentDocument);

        if (!$result) {
            return false;
        }
        
        $ancestors =  $this->getDocumentAncestors();
        
        if (!$ancestors) {
            return true;
        }
        
        foreach ($ancestors as $value) {
            $result = $this->_isAvailableForDisplay($value);
        
            if (!$result) {
                return false;
            }
        }
        
        return true;  
    } //_isAvailableforTheCurrentVersion
    
    private function _isAvailableForDisplay($documentId)
    {
        return !$this->getDocumentVersions($documentId)
               || $this->_hasCurrentVersion($documentId);
    } // end _isAvailableForDisplay
    
    public function getDocumentVersions($documentId)
    {
        $versions =  wp_get_post_terms($documentId, $this->_versions); 
        
        return $versions;  
    } // end getDocumentVersions
    
    private function _hasCurrentVersion($documentId)
    {
        $result = has_term(
            $this->_currentVersion,
            $this->_versions,
            $documentId
        );
        
        return $result;     
    } // end _hasCurrentVersion
    
    public function getCurrentPage()
    {
        $page = $this->getFrontendPage();

        return $page->ID;
    } // end getCurrentPage
    
    public function onDocumentEditLinkFilter($url)
    {
        $url = get_admin_url();
        $url .= 'post.php?post='.$this->_currentDocument.'&action=edit';
        return $url;
    } // end onDocumentEditLinkFilter
    
    public function onRemoveDocumentEditLinkFilter($url)
    {
        $url = get_admin_url();
        $url .= 'edit.php?post_type='.$this->_documents;
        return $url;
    } // end onRemoveDocumentEditLinkFilter
    
    public function getCurrentDocument()
    {
        if (!$this->hasDocumentInRequest()) {
            return 0;
        }

        $slug = $this->getValueFromRequest($this->_documentKey);
        
        $currentDocument = $this->getDocumentOptionByField(
            'name',
             $slug,
             'ID'
        );
        
        if ($currentDocument === false) {
            $currentDocument = 0;
        }

        return $currentDocument;
    } // end getCurrentDocument
    
    public function getCurrentVersion()
    {
        if (!$this->_allVersions) {
            return false;
        }
        
        if ($this->hasVersionInRequest()) {
            
            $slug = $this->getValueFromRequest($this->_versionKey);

            $version = $this->getVersionOptionByField(
                'slug',
                $slug,
                'term_id'
            );

            if (!$version) {
                return false;
            }
            
            if (in_array($version, $this->_allVersions)) {           
                return $version;
            }
        }

        $version = current($this->_allVersions);

        return $version;
    } // end getCurrentVersion
    
    public function onDisplayContentAction($content)
    {
        $postId = get_the_ID();
        
        if (!$this->_isFrontendPage($postId, $this->_currentPage)) {
           return  $content;
        }

        echo $this->fetchBreadCrumbs();
        
        if (!$this->_currentDocument) {
            echo $content;
            $content = '';
        }     
        
        $vars = array(
            'content' => $this->fetchHeader()
        );
        
        echo $this->fetch('header.phtml',$vars);

        $docs = $this->getDocumentsTree();
        
        if ($this->_hasTopLocationInOptions()) {
            echo $docs.$content; 
        } else {
            echo $content.$docs;  
        }
    } // end onDisplayContentAction
    
    private function _hasTopLocationInOptions()
    {
      return $this->_options['docsLocation'] == 'top';
    } // end _hasTopLocationInOptions
    
    private function _doRedirect($version = true)
    {
        $url = $this->_getDocumentUrl(false, $version);
        
        $vars = array(
            'url' => $url
        );
                        
        echo $this->fetch('redirect.phtml', $vars);
        
        exit();
    } // end _doRedirectToThePageOfTheCurrentVersion
    
    public function getDocumentsTree()
    {    
        $content = $this->fetchDocumentsTree();
        
        if (!$content && !$this->hasDocumentInRequest()) {
            
            $message = __(
                'There is no document for the current version',
                $this->_languageDomain
            );
            
            $vars = array('message' => $message);
            
            return $this->fetch('info_message.phtml', $vars);
         }
        
         return  $content;
    } // end getDocumentsTree
    
    public function fetchDocumentsTree()
    {
        $content = $this->fetchTheBranchesOfTree(0, $this->_currentDocument);
        
        if (!$content) {
            return false;
        }
        
        $vars = array(
            'content' => $content
        );
        
        return $this->fetch('list.phtml', $vars);
        
    } // end fetchDocumentsTree()
    
    public function fetchTheBranchesOfTree($level = 0, $documentId)
    {
        //$excludedVersions = $this->getExcludedVersions();
                       
        $margin = 30;
        $documents = array();
        $content = '';

        $result = $this->getDocumentsList($documentId);

        if ($result) {
            $documents = $result;
        }
                
        if (!$documents) {
            return $content;
        }
        
        $marginLeft = $margin * $level;
        $level++;

        foreach ($documents as $document) {
            if (!$this->_isAvailableForDisplay($document->ID)) {
                continue;
            }

            $url = $this->_getDocumentUrl($document->post_name);
            
            $vars = array(
                'caption'    => $document->post_title,
                'marginLeft' => $marginLeft,
                'url'        => $url
            );
            
            $content .=  $this->fetch('list_item.phtml', $vars);
            
            $documentId = $document->ID;
            
            $content .= $this->fetchTheBranchesOfTree($level, $documentId);
        }
        
        return  $content;
    } // end fetchTheBranchesOfTree
    
    public function getExcludedVersions()
    {
        $curentVersion = array($this->_currentVersion);
        
        $excludedVersions = array_diff($this->_allVersions, $curentVersion);
        
        return $excludedVersions;  
    } // end geExcludedVersions
    
    public function getDocumentsList($parentId)
    {
        $orderBy = $this->_options['docsOrderBy'];
        
        $order = $this->_options['docsOrder'];

        $postTypeArgs = array(
            'post_parent' => $parentId,
            'post_type'   => $this->_documents,
            'numberposts' => -1,
            'orderby'     => $orderBy,
            'order'       => $order,
        );

        return get_posts($postTypeArgs);      
        
    } // end getDocumentsList
    
    public function getDocumentContent()
    {
        if (!$this->_currentDocument && $this->hasDocumentInRequest()) {
            $message = __(
                'Not Found Document Which in the Link',
                    $this->_languageDomain
                );
             
            $vars = array('message' => $message);
            
            return $this->fetch('info_message.phtml', $vars);
        }
        
        if (!$this->_currentDocument) {
            return false;
        }
        
        if (!$this->_isAvailableToTheUser()) {
            $message = __(
                'This document is available only to registered users',
                $this->_languageDomain
            );
            
            $vars = array('message' => $message);
            
            return $this->fetch('info_message.phtml', $vars); 
        }

        $content = $this->getDocumentOptionByField(
            'post__in',
            array($this->_currentDocument),
            'post_content'
        );
        
        return $content;
    } // end getDocumentContent
    
    private function _isAvailableToTheUser()
    {
        $currentUser = wp_get_current_user();
        
        $access = $this->getMetaOptions(
            $this->_currentDocument,
            'festi_docs_access_parameters',
            true
        );

        return !$access || ($access && $currentUser->ID != 0); 
    } // end _isAvailableToTheUser
    
    public function fetchHeader($vars = array()) {  
        if ($this->_currentDocument) {
            $header = $this->getDocumentOptionByField(
                'post__in',
                array($this->_currentDocument),
                'post_title'
            );
            
            return $header;
        }
        
        if ($this->_currentVersion) {
            $header = $this->getVersionOptionByField(
                'id',
                $this->_currentVersion,
                'name'
            );
            
            $content = $this->_generalPageTitle.' '.$header;
            
        } else {
            $content = $this->_generalPageTitle;
        }
        
        $content .= ' '.__("documentation", $this->_languageDomain);
        
        return $content;
    } // end fetchHeader
    
    public function fetchBreadCrumbs()
    {
        $breadCrumbs = array();

        $breadCrumb = $this->_getDocumentsBreadCrumb();
        if ($breadCrumb) {
            $breadCrumbs = $breadCrumb;
        }
  
        $breadCrumb = $this->_getGeneralPageBreadCrumb();
        if ($breadCrumb) {
            $breadCrumbs[] = $breadCrumb;
        }
        
        $breadCrumb = $this->_getVersionBreadCrumb();
        if ($breadCrumb) {
            $breadCrumbs = array_merge($breadCrumbs, $breadCrumb);
        }
        
        $breadCrumb = $this->_getHomePageBreadCrumb();
        if ($breadCrumb) {
            $breadCrumbs[] = $breadCrumb;
        }

        $breadCrumbs = array_reverse($breadCrumbs);

        $vars = array(
            'breadCrumbs' => $breadCrumbs
        );
        
        $content = $this->fetch('bread_crumbs_box.phtml', $vars);
        
        return $content; 
    } // end fetchBreadCrumbs
    
    private function _getGeneralPageBreadCrumb()
    {
        $breadCrumb = array();
        
        $breadCrumb['title'] = __("Documentation", $this->_languageDomain);
        
        if (!$this->_hasMoreThanOneVersion() && !$this->_currentDocument) {
            return $breadCrumb;
        }

        $url = $this->_getDocumentUrl(false);
        
        $breadCrumb['link'] = $url;
        
        if (!$this->_currentDocument) {          
            $breadCrumb['thanSymbol'] = false;   
        }
        
        return $breadCrumb;
    } // end _getPostTypeBreadCrumb
    
    private function _getHomePageBreadCrumb()
    {
        $breadCrumb = array();
        
        $breadCrumb['title'] = __("Home", $this->_languageDomain);
       
        $url = get_site_url();
        
        $breadCrumb['link'] = $url;
        
        return $breadCrumb;
    } // end _getHomePageBreadCrumbb

    
    private function _getVersionBreadCrumb()
    {
        if (!$this->_allVersions) {
            return false;
        }
        
        $versions = $this->_allVersions;
        
        if (!$this->_hasMoreThanOneVersion()) {
            return false;
        }

        $versionsList = array();
        
        foreach ($this->_allVersions as $value) {
            $title = $this->getVersionOptionByField(
                'id',
                $value,
                'name'
            );
            
            if (!$title) {
                return false;
            }
            
            $slug = $this->getVersionOptionByField(
                'id',
                $value,
                'slug'
            );
            
            if (!$slug) {
                return false;
            }
            
            $versionsList[$value]['caption'] = $title;
            $versionsList[$value]['slug'] = $slug;
        }
        
        $vars = array(
            'current'  => $this->_currentVersion ,
            'versions' => $versionsList,
            'links'    => $this->_getVersionsLinks($versionsList)
        );

        $content = $this->fetch('versions_select.phtml', $vars);
        
        $breadCrumb[]= array(
            'title' => $content,
            'type'  => 'select'
        );
        
        return $breadCrumb;
    } // end _getVersionBreadCrumb
    
    private function _getVersionsLinks($versions = array())
    {
        if (!$versions) {
            return false;
        }
        
        $links = array();
        
        foreach ($versions as $key => $value) {
            $links[$key] = $this->_getVersionUrl($value['slug']);
        }
        
        return $links;
    } // end _getVersionsLinks
    
    private function _getVersionUrl($slug)
    {
        $pageId = $this->_currentPage;
        
        if ($this->hasDocumentInRequest()) {           
            $document = $this->getValueFromRequest($this->_documentKey);  
        } else {
            $document = false;
        }
        
        if ($this->hasPreviewInRequest()) {           
            $preview = true;  
        } else {
            $preview = false;
        }
        
        return $this->getLink($pageId, $slug, $document, $preview);
    } // end _getDocumentUrl
    
    private function _getDocumentsBreadCrumb()
    {
        if (!$this->_currentDocument) {
            return false;
        }

        $title = $this->getDocumentOptionByField(
            'post__in',
            array($this->_currentDocument),
            'post_title'
        );

        if (!$title) {
            return false;
        }
        
        $level = 0;
        
        $breadCrumbs = array();
        
        $breadCrumbs[$level] = array(
            'title' => $title
        );

        $ancestorsBreadCrumbs =  $this->_getDocumentAncestorsBreadCrumbs();
        
        if (!$ancestorsBreadCrumbs) {
            return $breadCrumbs;
        }
        
        $breadCrumbs = array_merge($breadCrumbs, $ancestorsBreadCrumbs);

        return $breadCrumbs;
    } //end getCategoriesBreadCrumbs
    
    private function _getDocumentAncestorsBreadCrumbs()
    {
        $ancestors =  $this->getDocumentAncestors();

        if (!$ancestors) {
            return false;
        }
        
        $breadCrumbs = array();
        $level = 0;

        foreach ($ancestors as $value) {
            
            $title = $this->getDocumentOptionByField(
                'post__in',
                array($value),
                'post_title'
            );
            
            $slug  = $this->getDocumentOptionByField(
                'post__in',
                array($value),
                'post_name'
            );

            $url = $this->_getDocumentUrl($slug);
            
            $level++;
            
            $breadCrumbs[$level]['title'] = $title;
            $breadCrumbs[$level]['link']  = $url;
        }

        return $breadCrumbs;
        
    } // end _getDocumentAncestorsBreadCrumbs
    
    private function _getDocumentUrl($slug, $withVersion = true)
    {
        $pageId = $this->_currentPage;
        
        if ($this->hasVersionInRequest() && $withVersion) {           
            $version = $this->getValueFromRequest($this->_versionKey);  
        } else {
            $version = false;
        }
        
        return $this->getLink($pageId, $version, $slug);
    } // end _getDocumentUrl
    
    public function getDocumentAncestors()
    {
        return get_ancestors($this->_currentDocument, $this->_documents);

    } // end getDocumentAncestors
    
    public function _hasPostObjectInPage()
    {
        return get_post();
    } // end _hasPostObjectInPage
    
    private function _hasMoreThanOneVersion()
    {
        if (!$this->_allVersions) {
            return false;
        }
        
        $count = count($this->_allVersions);
        return  $count > 1;
    } // end _hasMoreThanOneVersion
    
    public function hasVersionInRequest()
    {
        $query = $this->getWpQueryInstance();
        return array_key_exists($this->_versionKey, $query)
               && !empty($query[$this->_versionKey]);
    } // end _hasVersionInRequest
    
    public function hasDocumentInRequest()
    {
        $query = $this->getWpQueryInstance();
        return array_key_exists($this->_documentKey, $query)
               && !empty($query[$this->_documentKey]); 
    } // end hasDocumentInRequest
    
    public function hasPreviewInRequest()
    {
        $query = $this->getWpQueryInstance();
        return array_key_exists('festi_docs_preview', $query)
               && $query['festi_docs_preview'] == 'true'; 
    } // end hasDocumentInRequest
    
    public function &getWpQueryInstance()
    {
        if (!$this->_hasWordpressQueryInGlobals()) {
            return false;
        }
        
        $wpQuery = $GLOBALS['wp_query'];
        
        $keys = array(
            $this->_documentKey  => false,
            $this->_versionKey   => false,
            'festi_docs_preview' => false
        );

        $diff = array_diff_key($wpQuery->query_vars, $keys);
        $query = array_diff_key($wpQuery->query_vars, $diff);

        $query = array_merge($query, $_GET);

        return $query;
    } // end getWoocommerceInstance
    
    public function getValueFromRequest($key)
    {
        $query = $this->getWpQueryInstance();
        
        return $query[$key];
        
    } // end getValueFromRequest
    
    private function _hasWordpressQueryInGlobals()
    {
        return array_key_exists('wp_query', $GLOBALS);
    } // end _hasWordpressQueryInGlobals
    
    public function getDocumentOptionByField($field, $value, $option)
    {
        if (!is_array($value)) {
            $value = trim($value);
        }
        
        $args = array(
            'post_type'   => $this->_documents,
            'post_status' => 'publish',
            $field        => $value
        );

        $result = get_posts($args);

        if (!$result) {
            return false;    
        }
        
        $value = wp_list_pluck($result, $option);
        
        return $value[0];
    } // end getDocumentOptionByField
}


