<?php
/**
 * Plugin Name: WordPress Documentation Builder
 * Plugin URI: http://festi.io/app/festi-docs-plugin/
 * Description:  Documentation Builder is a WordPress plugin for creating Users Manuals, Instructions, Documentation.
 * Version: 1.2
 * Author: Festi 
 * Author URI: http://festi.io/
 */

if (!class_exists('FestiPlugin')) {
    require_once dirname(__FILE__).'/common/FestiPlugin.php';
}

class DocsFestiPlugin extends FestiPlugin
{
    protected $_languageDomain = 'festi_docs';
    protected $_optionsPrefix = 'festi_docs_';
    protected $_documents = 'festi_docs_documents';
    protected $_versions = 'festi_docs_versions';
    protected $_documentKey = 'festi_docs_document';
    protected $_versionKey = 'festi_docs_version'; 
    
    protected function onInit()
    {
        parent::onInit();
        
        if (defined('DOING_AJAX')) {
            $this->onBackendInit();
        }
        
        $this->addActionListener('plugins_loaded', 'onLanguagesInitAction');

        if ($this->hasFrontendPageInWordpress()) { 
            $this->addActionListener(
                'init',
                'onRegisterNewListOfPostsForDocumentsAction'
            );

            $this->addActionListener(
                'init',
                'onRegisterNewTaxonomiesForVersionsOfDocumentationAction'
            );
        }       
        
        if ($this->isEnablePermalinkOption()) {         
            $this->addActionListener(
                'init',
                'onRewriteRulesAction'
            ); 
        }
    } // end onInit
    
    public function onRewriteRulesAction()
    {
        $page = $this->getFrontendPage();
        
        $pageId = $page->ID;

        $pageRegExp = $page->post_name;
        
        $versionRegExp = 'version/([^/]*)';
        
        $documentRegExp = 'document/([^/]*)';
        
        $view = 'preview';
        
        $regExp = array(
            $pageRegExp.'/'.$versionRegExp.'/?$',
            $pageRegExp.'/'.$documentRegExp.'/?$',
            $pageRegExp.'/'.$documentRegExp.'/'.$view.'/?$',
            $pageRegExp.'/'.$versionRegExp.'/'.$documentRegExp.'/?$',
            $pageRegExp.'/'.$versionRegExp.'/'.$documentRegExp.'/'.$view.'/?$',
        );

        $rules = array(
            $regExp[0] => $this->_getQueryString($pageId, true, false),
            $regExp[1] => $this->_getQueryString($pageId, false, true),
            $regExp[2] => $this->_getQueryString($pageId, false, true, true),
            $regExp[3] => $this->_getQueryString($pageId, true, true),
            $regExp[4] => $this->_getQueryString($pageId, true, true, true)
        );

        $this->_appendKeyToWordpressQueryAction($this->_versionKey);
        $this->_appendKeyToWordpressQueryAction($this->_documentKey);
        $this->_appendKeyToWordpressQueryAction('festi_docs_preview');

        foreach ($rules as $key => $value) {
             add_rewrite_rule($key, $value, 'top'); 
        }
    } // end onRewriteRulesAction

    private function _appendKeyToWordpressQueryAction($key)
    {
        add_rewrite_tag('%'.$key.'%','([^&]+)'); 
    } // end _appendKeyToWordpressQuery

    private function _getQueryString($pageId, $version, $doc, $preview = false)
    {
        $str = 'index.php?page_id='.$pageId;
        
        $n = 1;
        
        if ($version) {
           $str .= '&'.$this->_versionKey.'=$matches['.$n.']';
           $n++; 
        }
        
        if ($doc) {
           $str .= '&'.$this->_documentKey.'=$matches['.$n.']';
        }
        
        if ($preview) {
           $str .= '&festi_docs_preview=true';
        }

        return $str;
    } // end _getQueryString

    public function onLanguagesInitAction()
    {        
        load_plugin_textdomain(
            $this->_languageDomain,
            false,
            $this->_pluginDirName.'/languages/'
        );
    } // end onLanguagesInitAction
    
    public function onInstall()
    {
        $plugin = $this->onBackendInit();
        
        $plugin->onInstall();
        
        $this->onRewriteRulesAction();
        
        flush_rewrite_rules();
    } // end onInstall
    
    protected function onFrontendInit()
    {
        require_once $this->_pluginPath.'common/DocsFrontendFestiPlugin.php';
        $frontend = new DocsFrontendFestiPlugin(__FILE__);
        return $frontend;
    } // end onFrontendInit
    
    protected function onBackendInit()
    {
        require_once $this->_pluginPath.'common/DocsBackendFestiPlugin.php';
        $backend = new DocsBackendFestiPlugin(__FILE__);
        return $backend;
    } // end onBackendInit
    
    public function onRegisterNewListOfPostsForDocumentsAction()
    {
        $domain = $this->_languageDomain;
        
        $lables = array(
            'name'               => __('Documentation', $domain),
            'singular_name'      => __('Document', $domain),
            'all_items'          => __('All Documents', $domain),
            'add_new'            => _x('New Document', $domain),
            'add_new_item'       => __('Add New Document', $domain),
            'edit_item'          => __('Edit Document', $domain),
            'new_item'           => __('New Document', $domain),
            'view_item'          => __('View Document', $domain),
            'search_items'       => __('Search Documents', $domain),
            'not_found'          => __('No Documents found', $domain),
            'not_found_in_trash' => __('No Documents found in trash', $domain)
        );
        
        $supports = array(
            'title',
            'editor',
            'thumbnail',
            'page-attributes',
        );
        
        $taxonomies = array(
            'festi_docs_versions'
        );
        
        $rewrite = array(
           'slug'               => $this->_documents,
           'with_front'         => false,
           'pages' => false
        );

        $args = array(
            'labels'            => $lables,
            'public'            => true,
            'show_ui'           => true,
            'hierarchical'      => true,
            'supports'          => $supports,
            'taxonomies'        => $taxonomies,
            'has_archive'       => true,
            'rewrite'           => $rewrite,
            'query_var'         => true,
            'show_in_nav_menus' => true,
            //'show_in_menu'  => 'festi-docs',
            'show_ui '          => true,
            'menu_icon'         => $this->getPluginImagesUrl('icon_16x16.png')
        );
        
        register_post_type($this->_documents, $args);
    } //end onRegisterNewListOfPostsForDocumentsAction
    
    public function onRegisterNewTaxonomiesForVersionsOfDocumentationAction()
    {
        $domain = $this->_languageDomain;
        
        $labels = array(
            'name'              => _x(
                'Versions',
                'taxonomy general name',
                $domain
            ),
            'singular_name'     => _x(
                'Version',
                'taxonomy singular name',
                $domain
            ),
            'search_items'      => __('Search Versions', $domain),
            'all_items'         => __('All Versions', $domain),
            'edit_item'         => __('Edit Version', $domain),
            'update_item'       => __('Update Version', $domain),
            'add_new_item'      => __('Add New Version', $domain),
            'new_item_name'     => __('New Version Name', $domain),
            'menu_name'         => __('Versions', $domain),
            'parent_item'         => __('Parent Versions', $domain),
        ); 
        
        $rewrite = array(
            'slug' => $this->_versions
        );
        
        $args = array(
            'label'             => __('Versions', $domain),
            'labels'            => $labels,
            'public'            => true,
            'show_in_nav_menus' => true,
            'show_ui'           => true,
            'show_tagcloud'     => false,
            'hierarchical'      => true,
            'rewrite'           => $rewrite,
            'query_var'         => true,
            'show_admin_column' => true
        );
        
        register_taxonomy(
            $this->_versions,
            array($this->_documents),
            $args
        );
    } // end onRegisterNewTaxonomiesForVersionsOfDocumentationAction
    
    public function getFrontendPage()
    {     
        $settings = $this->getOptions('settings');
        
        if (!$this->hasFrontendPageInSettings($settings))
        {
            return false;
        }
        
        $page = $this->getPageData($settings['page']);
        
        if (!$page) {
            return false;
        }
        
        $status = $page->post_status;
        
        if (!$this->_isPublicPage($status)) {
            return false;
        }
        
        return $page;
    } // end hasFrontendPageInWordpress
    
    private function _isPublicPage($status)
    {
        return $status == 'publish';
    } // _isPublicPage
    
    public function hasFrontendPageInSettings($settings =  array())
    {        
        return isset($settings)
               && array_key_exists('page', $settings)
               && !empty($settings['page']);
    } // end hasFrontendPageInSettings
    
    public function getPageData($pageId)
    {
        $data = get_page($pageId);
        return $data;
    } // end getFrontendPage

    public function getLink($pageId, $version, $document, $preview = false)
    {
        if ($this->_isWordpressFrontPage($pageId)) {
            $page = $this->getFrontendPage();
            $pageUrl = get_site_url().'/'.$page->post_name.'/';    
        } else {
            $pageUrl = get_page_link($pageId);
        }
        
        if (!$pageUrl) {
            return false;
        }

        if ($this->isEnablePermalinkOption()) {
            if ($version)
            {
                $pageUrl .= 'version/'.$version.'/';
            }
            
            if ($document)
            {
                $pageUrl .= 'document/'.$document.'/';
            }
            
            if ($preview) {
                $pageUrl .= 'preview/';
            }
        } else {
            if ($version)
            {
                $seperator = $this->getSeparator($pageUrl);
                $pageUrl .= $seperator.$this->_versionKey.'='.$version;
            }
            
            if ($document)
            {
                $seperator = $this->getSeparator($pageUrl);
                $pageUrl .= $seperator.$this->_documentKey.'='.$document;
            }
            
            if ($preview) {
                $seperator = $this->getSeparator($pageUrl);
                $pageUrl .= $seperator.'festi_docs_preview=true';
            }
        }

        return $pageUrl;
    } // end getLink
    
    public function getSeparator($url)
    {
        $seperator = preg_match("#\?#Umis", $url) ? '&' : '?';
        return $seperator;
    } // getSeparator
    
    public function isEnablePermalinkOption()
    {
        return get_option('permalink_structure');
    } // end isEnablePermalinkOption
    
    public function hasFrontendPageInWordpress()
    {
        $page = $this->getFrontendPage();
        
        return $page;
    } // end hasFrontendPageInWordpress
    
    public function getVersionOptionByField($field, $value, $option)
    {
       $value= trim($value);
        
       $result = get_term_by($field, $value, $this->_versions, ARRAY_A);
       
       return $result[$option];
    } // end getVersionOptionByField
    
    public function getMetaOptions($postId, $metaId, $single)
    {
        $value = get_post_meta($postId, '_'.$metaId.'_data', $single);
        
        return $value; 
    } // end getMetaOptions
    
    public function getVersionsListFromShortcodeVariable($versions)
    {
        $versions = trim($versions);
        
        if (!$versions) {
            return false;
        }
        
        $versions = explode(",", $versions);
   
        $versions = array_diff($versions, array(''));

        return $versions;
    } // end getVersionsListFromShortcodeVariable

    private function _isWordpressFrontPage($pageId)
    {
        $page = get_option('page_on_front');
        
        return $page == $pageId;
    } // end _isWordpressFrontPage
    
    public function getAllVersions($order = 'id', $sort = 'ASC', $field = 'all')
    {
        $args = array(
            'orderby'    => $order,
            'hide_empty' => false, 
            'order'      => $sort,
            'fields'     => $field
        );
        
        $versionsList = get_terms($this->_versions, $args);

        return $versionsList;
    } // end getAllVersions
    
    public function addFilterListener($hook, $method)
    {
        $priority = '';
        $acceptedArgs = '';
        
        $args = func_get_args();
        
        if (isset($args[2])) {
            $priority = $args[2];
        }
        
        if (isset($args[2])) {
            $acceptedArgs = $args[2];
        }
        
        add_filter($hook, array(&$this, $method), $priority, $acceptedArgs);
    } // end addActionListener
}

$GLOBALS['docsFestiPlugin'] = new DocsFestiPlugin(__FILE__);